@@ -19,8 +19,8 @@ require 'faraday/dependency_loader'
 #   conn.get '/'
 #
 module Faraday
-  VERSION = '0.16.2'
-  METHODS_WITH_QUERY = %w[get head delete connect trace].freeze
+  VERSION = '1.0.0'
+  METHODS_WITH_QUERY = %w[get head delete trace].freeze
   METHODS_WITH_BODY = %w[post put patch].freeze
 
   class << self
