@@ -14,7 +14,7 @@ module Faraday
   #
   class Connection
     # A Set of allowed HTTP verbs.
-    METHODS = Set.new %i[get post put delete head patch options trace connect]
+    METHODS = Set.new %i[get post put delete head patch options trace]
 
     # @return [Hash] URI query unencoded key/value pairs.
     attr_reader :params
@@ -99,7 +99,6 @@ module Faraday
         else
           proxy_from_env(url)
         end
-      @temp_proxy = @proxy
     end
 
     # Sets the Hash of URI query unencoded key/value pairs.
@@ -118,6 +117,13 @@ module Faraday
 
     def_delegators :builder, :build, :use, :request, :response, :adapter, :app
 
+    # Closes the underlying resources and/or connections. In the case of
+    # persistent connections, this closes all currently open connections
+    # but does not prevent new connections from being made.
+    def close
+      app.close
+    end
+
     # @!method get(url = nil, params = nil, headers = nil)
     # Makes a GET HTTP request without a body.
     # @!scope class
@@ -170,21 +176,6 @@ module Faraday
     # @yield [Faraday::Request] for further request customizations
     # @return [Faraday::Response]
 
-    # @!method connect(url = nil, params = nil, headers = nil)
-    # Makes a CONNECT HTTP request without a body.
-    # @!scope class
-    #
-    # @param url [String] The optional String base URL to use as a prefix for
-    #            all requests.  Can also be the options Hash.
-    # @param params [Hash] Hash of URI query unencoded key/value pairs.
-    # @param headers [Hash] unencoded HTTP header key/value pairs.
-    #
-    # @example
-    #   conn.connect '/items/1'
-    #
-    # @yield [Faraday::Request] for further request customizations
-    # @return [Faraday::Response]
-
     # @!method trace(url = nil, params = nil, headers = nil)
     # Makes a TRACE HTTP request without a body.
     # @!scope class
@@ -490,11 +481,8 @@ module Faraday
         raise ArgumentError, "unknown http method: #{method}"
       end
 
-      # Resets temp_proxy
-      @temp_proxy = proxy_for_request(url)
-
       request = build_request(method) do |req|
-        req.options = req.options.merge(proxy: @temp_proxy)
+        req.options.proxy = proxy_for_request(url)
         req.url(url)                if url
         req.headers.update(headers) if headers
         req.body = body             if body
@@ -514,7 +502,7 @@ module Faraday
       Request.create(method) do |req|
         req.params  = params.dup
         req.headers = headers.dup
-        req.options = options
+        req.options = options.dup
         yield(req) if block_given?
       end
     end
